<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns="http://www.hyfinity.com/schemas/tutorial" xmlns:demo="http://www.hyfinity.com/schemas/tutorial" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output encoding="UTF-8" indent="yes" method="xml" version="1.0" />

    <xsl:variable name="lcletters">abcdefghijklmnopqrstuvwxyz</xsl:variable>
    <xsl:variable name="ucletters">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>

    <xsl:variable name="listControl" select="/demo:ContactDB/demo:ListContactsRequest/demo:ListControl"/>
    <xsl:variable name="searchSurname" select="translate(/demo:ContactDB/demo:ListContactsRequest/demo:SearchCriteria/demo:ContactSurname, $ucletters, $lcletters)"/>
    <xsl:variable name="matchingContacts" select="/demo:ContactDB/demo:Contact[demo:IdentificationDetails/demo:Surname[$searchSurname = '' or contains(translate(., $ucletters, $lcletters), $searchSurname)]]"/>


    <xsl:template match="/">

        <xsl:variable name="pageSize">
            <xsl:choose>
                <xsl:when test="$listControl/demo:PageSize[. != '']">
                    <xsl:value-of select="$listControl/demo:PageSize" />
                </xsl:when>
                <xsl:otherwise>100</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="pageNum">
            <xsl:choose>
                <xsl:when test="$listControl/demo:PageNumber[. != '']">
                    <xsl:value-of select="$listControl/demo:PageNumber" />
                </xsl:when>
                <xsl:otherwise>1</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="startNum" select="($pageSize * ($pageNum - 1)) + 1"/>
        <xsl:variable name="endNum">
            <xsl:choose>
                <xsl:when test="count($matchingContacts) &lt; ($startNum + $pageSize - 1)">
                    <xsl:value-of select="count($matchingContacts)" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$startNum + $pageSize - 1"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>


        <demo:ListContactsResponse Version="1-0">
            <xsl:for-each select="$matchingContacts">
                <xsl:sort data-type="text" order="ascending" select="demo:IdentificationDetails/demo:Surname" />
                <xsl:sort data-type="text" order="ascending" select="demo:IdentificationDetails/demo:Forename" />

                <xsl:if test="(position() >= $startNum) and (position() &lt;= $endNum)">
                    <xsl:apply-templates select="."/>
                </xsl:if>
            </xsl:for-each>
            <demo:Validation>
                <demo:Errors>
                    <xsl:choose>
                        <xsl:when test="count($matchingContacts) > 0">
                            <demo:Error Cat="I" Code="1002" Count="1">Contacts List returned successfully</demo:Error>
                        </xsl:when>
                        <xsl:otherwise>
                            <demo:Error Cat="I" Code="1003" Count="1">No contacts match provided criteria</demo:Error>
                        </xsl:otherwise>
                    </xsl:choose>
                </demo:Errors>
            </demo:Validation>
            <demo:ListControl>
                <demo:PageSize><xsl:value-of select="$pageSize" /></demo:PageSize>
                <demo:PageNumber><xsl:value-of select="$pageNum" /></demo:PageNumber>
                <demo:StartNumber><xsl:value-of select="$startNum"/></demo:StartNumber>
                <demo:EndNumber>
                    <xsl:value-of select="$endNum"/>
                </demo:EndNumber>
                <demo:TotalSize>
                    <xsl:value-of select="count($matchingContacts)" />
                </demo:TotalSize>
                <demo:OrderListBy>Surname</demo:OrderListBy>
                <demo:OrderingType>Ascending</demo:OrderingType>
            </demo:ListControl>
        </demo:ListContactsResponse>
    </xsl:template>

    <xsl:template match="demo:Contact">
        <demo:ContactSummary>
            <demo:ContactId>
                <xsl:value-of select="@ContactId" />
            </demo:ContactId>
            <demo:Title>
                <xsl:value-of select="demo:IdentificationDetails/demo:Title" />
            </demo:Title>
            <demo:Forename>
                <xsl:value-of select="demo:IdentificationDetails/demo:Forename" />
            </demo:Forename>
            <demo:Surname>
                <xsl:value-of select="demo:IdentificationDetails/demo:Surname" />
            </demo:Surname>
            <demo:TelephoneNumber>
                <xsl:variable name="telNum" select="demo:ContactNumbers/demo:TelephoneNumbers/demo:TelephoneNumber[(@Preferred='yes') or ((demo:TelNationalNumber != '') and not(../demo:TelephoneNumber[@Preferred = 'yes']))][1]"/>
                <xsl:value-of select="$telNum/demo:TelCountryCode" />
                <xsl:if test="$telNum/demo:TelCountryCode[. != '']">
                    <xsl:text> - </xsl:text>
                </xsl:if>
                <xsl:value-of select="$telNum/demo:TelNationalNumber" />
            </demo:TelephoneNumber>
            <demo:EmailAddress>
                <xsl:variable name="email" select="demo:ContactNumbers/demo:EmailAddresses/demo:EmailAddress[(@Preferred='yes') or ((demo:Email != '') and not(../demo:EmailAddress[@Preferred = 'yes']))][1]"/>
                <xsl:value-of select="$email/demo:Email" />
            </demo:EmailAddress>
            <demo:MoreNumbers>
                <xsl:choose>
                    <xsl:when test="count(demo:ContactNumbers/demo:TelephoneNumbers/demo:TelephoneNumber/demo:TelNationalNumber[. != '']) &gt; 1">
                        <xsl:text>yes</xsl:text>
                    </xsl:when>
                    <xsl:when test="count(demo:ContactNumbers/demo:EmailAddresses/demo:EmailAddress/demo:Email[. != '']) &gt; 1">
                        <xsl:text>yes</xsl:text>
                    </xsl:when>
                    <xsl:when test="count(demo:ContactNumbers/demo:FaxNumbers/demo:FaxNumber/demo:FaxNationalNumber[. != '']) &gt; 0">
                        <xsl:text>yes</xsl:text>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>no</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </demo:MoreNumbers>
        </demo:ContactSummary>
    </xsl:template>
</xsl:stylesheet>
